package job

import (
	"context"
	"fmt"
	"time"

	"go.uber.org/zap"

	"github.com/freeman42332/zdb1000n/src/job/config"
	"github.com/freeman42332/zdb1000n/src/utils/metrics"
)

func tcpJob(ctx context.Context, args config.Args, globalConfig *GlobalConfig, a *metrics.Accumulator, logger *zap.Logger) (data any, err error) {
	return rawnetJob(ctx, "tcp", args, globalConfig, a, logger)
}

func udpJob(ctx context.Context, args config.Args, globalConfig *GlobalConfig, a *metrics.Accumulator, logger *zap.Logger) (data any, err error) {
	return rawnetJob(ctx, "udp", args, globalConfig, a, logger)
}

func rawnetJob(ctx context.Context, protocol string, args config.Args, globalConfig *GlobalConfig, a *metrics.Accumulator, logger *zap.Logger) (
	data any, err error,
) {
	packetgenArgs, err := parseRawNetJobArgs(logger, globalConfig, args, protocol)
	if err != nil {
		return nil, err
	}

	return packetgenJob(ctx, packetgenArgs, globalConfig, a, logger)
}

func parseRawNetJobArgs(logger *zap.Logger, globalConfig *GlobalConfig, args config.Args, protocol string) (
	result map[string]any, err error,
) {
	var jobConfig struct {
		BasicJobConfig

		Address   string
		Body      string
		ProxyURLs string
		Timeout   *time.Duration
	}

	if err := ParseConfig(&jobConfig, args, *globalConfig); err != nil {
		return nil, fmt.Errorf("error decoding rawnet job config: %w", err)
	}

	packetgenArgs := make(map[string]any)
	for k, v := range args {
		packetgenArgs[k] = v
	}

	packetgenArgs["connection"] = map[string]any{
		"type": "net",
		"args": map[string]any{
			"protocol":   protocol,
			"address":    jobConfig.Address,
			"timeout":    jobConfig.Timeout,
			"proxy_urls": jobConfig.ProxyURLs,
		},
	}
	packetgenArgs["packet"] = map[string]any{
		"payload": map[string]any{
			"type": "raw",
			"data": map[string]any{
				"payload": jobConfig.Body,
			},
		},
	}

	return packetgenArgs, nil
}
